@@ -1,5 +1,4 @@
 STF = function(sp, time) {
-	time[,1] = 1:nrow(time) # removes any old indexing
 	new("STF", ST(sp, time))
 }
 
@@ -26,10 +25,19 @@ index.STF = function(x, ...) {
 index.STFDF = index.STF
 
 as.data.frame.STF = function(x, row.names = NULL, ...) {
-  	data.frame(coordinates(x), 
+	if (is.null(row.names(x@sp)))
+		row.names(x@sp) = 1:nrow(x@sp)
+	timedata = apply(x@time, 2, rep, each = length(x@sp))
+  	ret = data.frame(coordinates(x), 
 		sp.ID = rep(row.names(x@sp), length(x@time)),
 		time = index(x),
+		timedata,
 		row.names = row.names, ...)
+	if ("data" %in% slotNames(x@sp)) {
+		df = data.frame(apply(x@sp@data, 2, rep, length(x@time)))
+		ret = data.frame(ret, df)
+	}
+	ret
 }
 setAs("STF", "data.frame", function(from) as.data.frame.STF(from))
 
@@ -70,6 +78,10 @@ subs.STFDF <- function(x, i, j, ... , drop = TRUE) {
 		x@data = x@data[ , k, drop = FALSE]
 		return(x)
 	} 
+
+	si = rep(1:length(x@sp), nrow(x@time))
+	ti = rep(1:nrow(x@time), each = length(x@sp))
+
 	if (missing.i)
 		s = 1:length(x@sp)
 	else {
@@ -78,19 +90,34 @@ subs.STFDF <- function(x, i, j, ... , drop = TRUE) {
 		else 
 			s = i
 	}
+	x@sp = x@sp[s,]
+
 	if (missing.j)
 		t = 1:nrow(x@time)
+	else {
+		nc = ncol(x@time)
+		x@time = cbind(x@time, 1:nrow(x@time))
+		# uses [.xts, deals with character/iso8601;
+		# takes care of negative indices:
+		x@time = x@time[j] 
+		# get back the corresponding index vector t, to use for @data:
+		t = x@time[, nc+1]
+		x@time = x@time[,-(nc+1)]
+	}
+
+	if (all(s < 0))
+		ssel = !(si %in% abs(s))
 	else
-		t = j
-	si = rep(1:length(x@sp), nrow(x@time))
-	ti = rep(1:nrow(x@time), each = length(x@sp))
-	x@sp = x@sp[s]
-	x@time = x@time[t] # uses [.xts, deals with character
-	t = x@time[,1] # gets the indices
-	x@time[,1] = 1:nrow(x@time) # resets indices
-	x@data = x@data[si %in% s & ti %in% t, k, drop = FALSE]
+		ssel = si %in% s
+
+	#if (all(t < 0))
+	#	tsel = !(ti %in% abs(t))
+	#else
+	tsel = ti %in% t
+
+	x@data = x@data[ssel & tsel, k, drop = FALSE]
 	if (drop) {
-		if (length(s) == 1) { # space index has only 1 item:
+		if (length(s) == 1 && all(s > 0)) { # space index has only 1 item:
 			if (length(t) == 1)
 				x = x@data[1,1,drop=TRUE]
 			else
@@ -101,3 +128,27 @@ subs.STFDF <- function(x, i, j, ... , drop = TRUE) {
 	x
 }
 setMethod("[", "STFDF", subs.STFDF)
+
+# provide a na.omit-method for STFDF objects
+# removes rows and columns from the space-time grid
+# containing NAs in the data
+# Tom Gottfried
+na.omit.STFDF <- function(object, drop=TRUE, ...){
+  data <- na.omit(object@data)
+  omit <- attr(data, "na.action")
+  n <- length(object@sp)
+  s <- unique((omit-1) %% n + 1)
+  t <- unique((omit-1) %/% n + 1)
+  if (drop && (length(s)==n || length(t)==nrow(object@time)))
+    return(NA)
+  else
+#    return(object[-s,-t, drop=drop])
+              # <= negative indices are partly not handled by [-method
+#    return(object[(1:n)[!(1:n) %in% s],
+#                  (1:nrow(object@time))[!1:nrow(object@time) %in% t],
+#                  drop=drop])
+              # <= logical indices partly not handled by [-method
+    return(object[(1:n)[!(1:n) %in% s],
+                  (1:nrow(object@time))[!1:nrow(object@time) %in% t],
+                  drop=drop])
+}
